@@ -2449,7 +2449,8 @@ beach:
 }
 
 static GstBaseTsMuxPad *
-gst_base_ts_mux_find_best_pad (GstAggregator * aggregator, gboolean timeout)
+gst_base_ts_mux_find_best_pad (GstAggregator * aggregator,
+    GstClockTime * best_time, gboolean timeout)
 {
   GstBaseTsMuxPad *best = NULL;
   GstClockTime best_ts = GST_CLOCK_TIME_NONE;
@@ -2461,6 +2462,7 @@ gst_base_ts_mux_find_best_pad (GstAggregator * aggregator, gboolean timeout)
     GstBaseTsMuxPad *tpad = GST_BASE_TS_MUX_PAD (l->data);
     GstAggregatorPad *apad = GST_AGGREGATOR_PAD_CAST (tpad);
     GstBuffer *buffer;
+    GstClockTime ts;
 
     buffer = gst_aggregator_pad_peek_buffer (apad);
     if (!buffer) {
@@ -2471,21 +2473,29 @@ gst_base_ts_mux_find_best_pad (GstAggregator * aggregator, gboolean timeout)
       }
       continue;
     }
-    if (best_ts == GST_CLOCK_TIME_NONE) {
+
+    ts = GST_BUFFER_DTS_OR_PTS (buffer);
+    if (!GST_CLOCK_TIME_IS_VALID (ts)) {
+      GST_WARNING_OBJECT (apad, "Buffer has no timestamp: %" GST_PTR_FORMAT,
+          buffer);
       best = tpad;
-      best_ts = GST_BUFFER_DTS_OR_PTS (buffer);
-    } else if (GST_BUFFER_DTS_OR_PTS (buffer) != GST_CLOCK_TIME_NONE) {
-      GstClockTime t = GST_BUFFER_DTS_OR_PTS (buffer);
-      if (t < best_ts) {
-        best = tpad;
-        best_ts = t;
-      }
+      best_ts = ts;
+      gst_buffer_unref (buffer);
+      break;
+    }
+
+    if (!GST_CLOCK_TIME_IS_VALID (best_ts) || ts < best_ts) {
+      best = tpad;
+      best_ts = ts;
     }
     gst_buffer_unref (buffer);
   }
 
-  if (best)
+  if (best) {
     gst_object_ref (best);
+    if (best_time)
+      *best_time = best_ts;
+  }
 
   GST_OBJECT_UNLOCK (aggregator);
 
@@ -2524,7 +2534,7 @@ gst_base_ts_mux_aggregate (GstAggregator * agg, gboolean timeout)
 {
   GstBaseTsMux *mux = GST_BASE_TS_MUX (agg);
   GstFlowReturn ret = GST_FLOW_OK;
-  GstBaseTsMuxPad *best = gst_base_ts_mux_find_best_pad (agg, timeout);
+  GstBaseTsMuxPad *best = gst_base_ts_mux_find_best_pad (agg, NULL, timeout);
   GstCaps *caps;
 
   /* set caps on the srcpad if no caps were set yet */
@@ -2598,6 +2608,21 @@ gst_base_ts_mux_stop (GstAggregator * agg)
   return TRUE;
 }
 
+static GstClockTime
+gst_base_ts_mux_get_next_time (GstAggregator * agg)
+{
+  GstBaseTsMuxPad *best = NULL;
+  GstClockTime next_time = GST_CLOCK_TIME_NONE;
+
+  best = gst_base_ts_mux_find_best_pad (agg, &next_time, TRUE);
+  // Buffer without timestamps are muxed immediately
+  if (best && next_time == GST_CLOCK_TIME_NONE)
+    next_time = 0;
+  gst_clear_object (&best);
+
+  return next_time;
+}
+
 /* GObject implementation */
 
 static void
@@ -2842,7 +2867,7 @@ gst_base_ts_mux_class_init (GstBaseTsMuxClass * klass)
   gstagg_class->src_event = gst_base_ts_mux_src_event;
   gstagg_class->start = gst_base_ts_mux_start;
   gstagg_class->stop = gst_base_ts_mux_stop;
-  gstagg_class->get_next_time = gst_aggregator_simple_get_next_time;
+  gstagg_class->get_next_time = gst_base_ts_mux_get_next_time;
 
   klass->create_ts_mux = gst_base_ts_mux_default_create_ts_mux;
   klass->allocate_packet = gst_base_ts_mux_default_allocate_packet;
